<?php

namespace catchAdmin\tag_history\controller;

use catchAdmin\report\model\Report;
use catcher\base\CatchRequest as Request;
use catcher\CatchResponse;
use catcher\base\CatchController;
use catchAdmin\tag_history\model\TagHistory as tagHistoryModel;
use catcher\CatchAdmin;
use catcher\Code;
use think\facade\Db;

class TagHistory extends CatchController
{
    protected $tagHistoryModel;

    public function __construct(TagHistoryModel $tagHistoryModel)
    {
        $this->tagHistoryModel = $tagHistoryModel;
    }

    /**
     * 列表
     * @time 2022年08月12日 11:31
     * @param Request $request 
     */
    public function index(Request $request): \think\Response
    {

        $data = $request->get();
        $time =  isset($data['time']) ? $data['time'] : '';
        $mac = isset($data['mac']) ? $data['mac'] : '';
        $tage = isset($data['tage']) ? $data['tage'] : '';
        $endtime = isset($data['endtime']) ? $data['endtime'] : "";
        $meter = isset($data['meter']) ? $data['meter'] : "";
        $where = [];
        $create_id = $request->user()->id;

        $bool = Db::name('user_has_roles')->where('uid', $create_id)->where('role_id', 3)->find();
        $shifouRole = Db::name('user_has_roles')->where('uid', $create_id)->where('role_id', 4)->find();



        if (!empty($time)) {
            $where[] = ['addTime', '>=', $time];
        } else {
            $where[] = ['addTime', '>=', time()];
        }

        if (!empty($mac)) {
            $where[] = ['mac', 'in',  implode(",", $mac)];
        }
        if (!empty($tage)) {
            $where[] = ['lable', 'in',  implode(",", $tage)];
        }
        if ($endtime == 1) {
            //生成报告
            $end = time();
            if (empty($mac)) {
                $mac = $this->tagHistoryModel->group('mac')->column('mac');
            }
            if (empty($tage)) {
                $tage = $this->tagHistoryModel->group('lable')->column('lable');
            }
            $tagList =  $this->tagHistoryModel->where($where)->where('addTime', '<=', $end)->select();
            $content = [
                'mac' => json_encode($mac),
                'lable' => json_encode($tage),
                'meter' => $meter,
                'startTime' => $time,
                'endTime' => $end,
                'data' => json_encode($tagList)
            ];
            $report = new Report();
            $report->save($content);
            return CatchResponse::success('生成报告成功');
        }

        if ($bool) {
            $where[] = ['mac', 'in', '30B5F1013899'];
            $where[] = ['lable', 'in', "3543245626,3544976938,3544261098,1539178988,3544517098,3544959402,3542053498,3543226346,3544413306"];
        } elseif ($shifouRole) {
            $where[] = ['mac', 'in', '30B5F10138EB'];
            $where[] = ['lable', 'in', "30b5f6000006,30b5f6000007,30b5f6000008"];
        } else {
            $where[] = ['mac', 'not in', '30B5F1013899'];
            $where[] = ['lable', 'not in', "3543245626,3544976938,3544261098,1539178988,3544517098,3544959402,3542053498,3543226346,3544413306"];
        }



        $list = $this->tagHistoryModel->group('mac,lable')->where($where)
            ->field('mac,lable,max(addTime) as addTime,format(AVG(rssi),2) as ave_rssi,count(*) as num')->select()->toArray();

        foreach ($list as $key => $value) {
            $wheres = [];
            $da = null;
            $wheres[] = ['mac', '=', $value['mac']];
            $wheres[] = ['lable', '=', $value['lable']];
            $wheres[] = ['addTime', '=', $value['addTime']];
            $da = $this->tagHistoryModel->where($wheres)->find();
            $list[$key]['rssi'] = $da['rssi'];
            $list[$key]['addTime'] = date('Y-m-d H:i:s', $value['addTime']);
            $list[$key]['time'] = date('Y-m-d H:i:s', $da['time']);
            $list[$key]['id'] = $da['move'];
        }
        return CatchResponse::success($list);
    }

    /**
     * 保存信息
     * @time 2022年08月12日 11:31
     * @param Request $request 
     */
    public function save(Request $request): \think\Response
    {
        return CatchResponse::success($this->tagHistoryModel->storeBy($request->post()));
    }

    /**
     * 读取
     * @time 2022年08月12日 11:31
     * @param $id 
     */
    public function read($id): \think\Response
    {
        return CatchResponse::success($this->tagHistoryModel->findBy($id));
    }

    /**
     * 更新
     * @time 2022年08月12日 11:31
     * @param Request $request 
     * @param $id
     */
    public function update(Request $request, $id): \think\Response
    {
        return CatchResponse::success($this->tagHistoryModel->updateBy($id, $request->post()));
    }

    /**
     * 删除
     * @time 2022年08月12日 11:31
     * @param $id
     */
    public function delete($id): \think\Response
    {
        return CatchResponse::success($this->tagHistoryModel->deleteBy($id));
    }

    /**
     * @Descripttion: 获取设备
     * @name: likang
     * @return {*}
     */
    public function getStations(Request $request)
    {
        $create_id = $request->user()->id;
        $where = [];
        $bool = Db::name('user_has_roles')->where('uid', $create_id)->where('role_id', 3)->find();
        $shifouRole = Db::name('user_has_roles')->where('uid', $create_id)->where('role_id', 4)->find();

        if ($bool) {
            $where[] = ['mac', 'in', '30B5F1013899'];
        } else if ($shifouRole) {
            $where[] = ['mac', 'in', '30B5F10138EB'];
        } else {
            $where[] = ['mac', 'not in', ['30B5F1013899', '30B5F10138EB']];
        }
        $list = $this->tagHistoryModel->group('mac')->where($where)->column('mac');
        return CatchResponse::success($list);
    }
    /**
     * @Descripttion: 获取标签
     * @name: likang
     * @return {*}
     */
    public function getTages(Request $request)
    {
        $create_id = $request->user()->id;
        $where = [];
        $bool = Db::name('user_has_roles')->where('uid', $create_id)->where('role_id', 3)->find();
        $shifouRole = Db::name('user_has_roles')->where('uid', $create_id)->where('role_id', 4)->find();

        if ($bool) {
            $where[] = ['lable', 'in', "3543245626,3544976938,3544261098,1539178988,3544517098,3544959402,3542053498,3543226346,3544413306"];
        } elseif ($shifouRole) {
            $where[] = ['lable', 'in', "30b5f6000006,30b5f6000007,30b5f6000008"];
        } else {
            $where[] = ['lable', 'not in', "3543245626,3544976938,3544261098,1539178988,3544517098,3544959402,3542053498,3543226346,3544413306,30b5f6000006,30b5f6000007,30b5f6000008"];
        }
        $list = $this->tagHistoryModel->group('lable')->where($where)->column('lable');
        return CatchResponse::success($list);
    }
    /**
     * @Descripttion: 获取时间
     * @name: likang
     * @return {*}
     */
    public function getTime()
    {

        $time = time();
        return CatchResponse::success($time);
    }
    /**
     * @Descripttion: 根据标签和设备获取数据和心跳
     * @name: likang
     * @param {Request} $request
     * @return {*}
     */
    public function getRecordByTag(Request $request)
    {
        $lable = $request->get('lable');
        $mac = $request->get('mac');
        $limit = 10;
        $page = 1;
        if ($request->get('page')) {
            $page = $request->get('page');
        }
        if ($request->get('limit')) {
            $limit = $request->get('limit');
        }
        $where = [
            ['lable', '=', $lable],
            ['mac', '=', $mac]
        ];
        $list = $this->tagHistoryModel->where($where)->order('addTime desc')->page($page)->limit($limit)->select();
        foreach ($list as $key => $value) {
            $list[$key]['time'] = date('Y-m-d H:i:s', $value['time']);
            $list[$key]['addTime'] = date('Y-m-d H:i:s', $value['addTime']);
        }
        return json([
            'code'    => Code::SUCCESS,
            'message' => 'success',
            'count'   => $this->tagHistoryModel->where($where)->count(),
            'current' => $page,
            'limit'   => $limit,
            'data'    => $list,
        ]);
    }
}
